from Classification.Attribute.DiscreteAttribute import DiscreteAttribute
from MorphologicalAnalysis.MorphologicalParse import MorphologicalParse


class CaseAttribute(DiscreteAttribute):

    def __init__(self, parse: MorphologicalParse):
        """
        Discrete attribute for a given word. If the last inflectional group of the word contains case information, the
        attribute will have that case value. Otherwise the attribute will have the value null.

        PARAMETERS
        ----------
        parse : MorphologicalParse
            Morphological parse of the word.
        """
        super().__init__(parse.lastIGContainsCase())



from Classification.Attribute.BinaryAttribute import BinaryAttribute
from MorphologicalAnalysis.MorphologicalParse import MorphologicalParse
from MorphologicalAnalysis.MorphologicalTag import MorphologicalTag


class LastIGContainsTagAttribute(BinaryAttribute):

    def __init__(self, parse: MorphologicalParse, tag: MorphologicalTag):
        """
        Binary attribute for a given word. If the last inflectional group of the word contains tag,
        the attribute will be "true", otherwise "false".

        PARAMETERS
        ----------
        parse : MorphologicalParse
            Morphological parse of the word.
        tag : MorphologicalTag
            Tag that is checked in the last inflectional group.
        """
        super().__init__(parse.lastIGContainsTag(tag))



from Classification.Attribute.BinaryAttribute import BinaryAttribute
from MorphologicalAnalysis.MorphologicalParse import MorphologicalParse


class IsAuxiliaryVerbAttribute(BinaryAttribute):

    def __init__(self, parse: MorphologicalParse):
        """
        Binary attribute for a given word. If the word is an auxiliary verb, the attribute will have
        the value "true", otherwise "false".

        PARAMETERS
        ----------
        parse : MorphologicalParse
            Morphological parse of the word.
        """
        super().__init__(parse.isAuxiliary() and parse.isVerb())



from AnnotatedSentence.AnnotatedSentence import AnnotatedSentence
from Classification.Attribute.DiscreteAttribute import DiscreteAttribute
from Corpus.Sentence import Sentence


class Predicate(DiscreteAttribute):

    def __init__(self, sentence: Sentence, index: int):
        """
        Discrete attribute for a given word. Returns the nearest predicate word to the given word

        PARAMETERS
        ----------
        sentence : Sentence
            Sentence where current word is in.
        index : int
            Position of the current word in the sentence
        """
        if isinstance(sentence, AnnotatedSentence):
            sentence.getPredicate(index)



from abc import abstractmethod

from Classification.Instance.Instance import Instance
from Corpus.Sentence import Sentence


class InstanceGenerator:

    windowSize: int

    @abstractmethod
    def generateInstanceFromSentence(self, sentence: Sentence, wordIndex: int) -> Instance:
        pass



from abc import abstractmethod

from AnnotatedSentence.AnnotatedWord import AnnotatedWord
from Classification.Instance.Instance import Instance
from Corpus.Sentence import Sentence

from DataGenerator.InstanceGenerator.InstanceGenerator import InstanceGenerator


class DisambiguationInstanceGenerator(InstanceGenerator):

    @abstractmethod
    def addAttributesForPreviousWords(self, current: Instance, sentence: Sentence, wordIndex: int):
        pass

    @abstractmethod
    def addAttributesForEmptyWords(self, current: Instance, emptyWord: str):
        pass

    def generateInstanceFromSentence(self, sentence: Sentence, wordIndex: int) -> Instance:
        """
        Generates a single classification instance of the morphological disambiguation problem for the given word of the
        given sentence. If the word does not have a morphological parse, the method throws InstanceNotGenerated.

        PARAMETERS
        ----------
        sentence : Sentence
            Input sentence.
        wordIndex : int
            The index of the word in the sentence.

        RETURNS
        -------
        Instance
            Classification instance.
        """
        word = sentence.getWord(wordIndex)
        if isinstance(word, AnnotatedWord):
            current = Instance(word.getParse().getTransitionList())
            for i in range(self.windowSize):
                if wordIndex - self.windowSize + i >= 0:
                    self.addAttributesForPreviousWords(current, sentence, wordIndex - self.windowSize + i)
                else:
                    self.addAttributesForEmptyWords(current, "<s>")
            self.addAttributesForPreviousWords(current, sentence, wordIndex)
            return current



from AnnotatedSentence.AnnotatedWord import AnnotatedWord
from Classification.Instance.Instance import Instance
from Corpus.Sentence import Sentence
from NamedEntityRecognition.NamedEntityType import NamedEntityType

from DataGenerator.InstanceGenerator.SimpleWindowInstanceGenerator import SimpleWindowInstanceGenerator


class NerInstanceGenerator(SimpleWindowInstanceGenerator):

    def addAttributesForWords(self, current: Instance, sentence: Sentence, wordIndex: int):
        pass

    def addAttributesForEmptyWords(self, current: Instance, emptyWord: str):
        pass

    def generateInstanceFromSentence(self, sentence: Sentence, wordIndex: int) -> Instance:
        word = sentence.getWord(wordIndex)
        if isinstance(word, AnnotatedWord):
            classLabel = NamedEntityType.getNamedEntityString(word.getNamedEntityType())
            current = Instance(classLabel)
            self.addAttributes(current, sentence, wordIndex)
            return current



from Classification.Attribute.BinaryAttribute import BinaryAttribute


class IsCapitalAttribute(BinaryAttribute):

    def __init__(self, surfaceForm: str):
        """
        Binary attribute for a given word. If the starting letter of the word is capital, the attribute will have
        the value "true", otherwise "false".

        PARAMETERS
        ----------
        surfaceForm: str
            Surface form of the word.
        """
        super().__init__(surfaceForm[0:0].isupper())



from AnnotatedSentence.ViewLayerType import ViewLayerType
from AnnotatedTree.TreeBankDrawable import TreeBankDrawable
from NamedEntityRecognition.NERCorpus import NERCorpus


class NERCorpusGenerator:

    __treeBank: TreeBankDrawable

    def __init__(self, folder: str, pattern: str):
        """
        Constructor for the NERCorpusGenerator which takes input the data directory and the pattern for the
        training files included. The constructor loads the treebank from the given directory including the given files
        the given pattern.

        PARAMETERS
        ----------
        folder : str
            Directory where the treebank files reside.
        pattern : str
            Pattern of the tree files to be included in the treebank. Use "." for all files.
        """
        self.__treeBank = TreeBankDrawable(folder, pattern)

    def generate(self) -> NERCorpus:
        """
        Creates a morphological disambiguation corpus from the treeBank. Calls generateAnnotatedSentence for each parse
        tree in the treebank.

        RETURNS
        -------
        DisambiguationCorpus
            Created disambiguation corpus.
        """
        corpus = NERCorpus()
        for i in range(self.__treeBank.size()):
            parseTree = self.__treeBank.get(i)
            if parseTree.layerAll(ViewLayerType.NER):
                sentence = parseTree.generateAnnotatedSentence()
                corpus.addSentence(sentence)
        return corpus



from Classification.Attribute.BinaryAttribute import BinaryAttribute
from Dictionary.Word import Word


class IsHonorificAttribute(BinaryAttribute):

    def __init__(self, surfaceForm: str):
        """
        Binary attribute for a given word. If the word is "bay" or "bayan", the attribute will have the value "true",
        otherwise "false".

        PARAMETERS
        ----------
        surfaceForm : str
            Surface form of the word.
        """
        super().__init__(Word.isHonorific(surfaceForm))



from Classification.Attribute.DiscreteAttribute import DiscreteAttribute
from MorphologicalAnalysis.MorphologicalParse import MorphologicalParse


class RootFormAttribute(DiscreteAttribute):

    def __init__(self, parse: MorphologicalParse):
        """
        Discrete attribute for a given word. Returns the the root word

        PARAMETERS
        ----------
        parse : MorphologicalParse
            Morphological parse of the word.
        """
        super().__init__(parse.getWord().getName())



from AnnotatedSentence.AnnotatedWord import AnnotatedWord
from Classification.Instance.Instance import Instance
from Corpus.Sentence import Sentence

from DataGenerator.InstanceGenerator.SimpleWindowInstanceGenerator import SimpleWindowInstanceGenerator


class ShallowParseInstanceGenerator(SimpleWindowInstanceGenerator):

    def addAttributesForWords(self, current: Instance, sentence: Sentence, wordIndex: int):
        pass

    def addAttributesForEmptyWords(self, current: Instance, emptyWord: str):
        pass

    def generateInstanceFromSentence(self, sentence: Sentence, wordIndex: int) -> Instance:
        """
        Generates a single classification instance of the Shallow Parse problem for the given word of the given
        sentence. If the  word has not been labeled with shallow parse tag yet, the method returns null.

        PARAMETERS
        ----------
        sentence : Sentence
            Input sentence.
        wordIndex : int
            The index of the word in the sentence.

        RETURNS
        -------
        Instance
            Classification instance.
        """
        word = sentence.getWord(wordIndex)
        if isinstance(word, AnnotatedWord):
            classLabel = word.getShallowParse()
            current = Instance(classLabel)
            self.addAttributes(current, sentence, wordIndex)
            return current



from Classification.Attribute.DiscreteAttribute import DiscreteAttribute
from MorphologicalAnalysis.MorphologicalParse import MorphologicalParse


class RootPosAttribute(DiscreteAttribute):

    def __init__(self, parse: MorphologicalParse):
        """
        Discrete attribute for a given word. Returns the part of speech of the root word

        PARAMETERS
        ----------
        parse : MorphologicalParse
            Morphological parse of the word.
        """
        super().__init__(parse.getRootPos())



from Classification.Instance.Instance import Instance
from Corpus.Sentence import Sentence

from DataGenerator.InstanceGenerator.InstanceGenerator import InstanceGenerator
from abc import abstractmethod


class SimpleWindowInstanceGenerator(InstanceGenerator):

    @abstractmethod
    def addAttributesForWords(self, current: Instance, sentence: Sentence, wordIndex: int):
        pass

    @abstractmethod
    def addAttributesForEmptyWords(self, current: Instance, emptyWord: str):
        pass

    def addAttributes(self, current: Instance, sentence: Sentence, wordIndex: int):
        """
        addAttributes adds all attributes of the previous words, the current wordn, and next words of the given word
        to the given instance. If the previous or next words does not exists, the method calls
        addAttributesForEmptyWords method. If the word does not exists in the dictionary or the required annotation
        layer does not exists in the annotated word, the method throws InstanceNotGenerated. The window size determines
        the number of previous and next words.

        PARAMETERS
        ----------
        current : Instance
            Current classification instance to which attributes will be added.
        sentence : Sentence
            Input sentence.
        wordIndex : int
            The index of the word in the sentence.
        """
        for i in range(self.windowSize):
            if wordIndex - self.windowSize + i >= 0:
                self.addAttributesForWords(current, sentence, wordIndex - self.windowSize + i)
            else:
                self.addAttributesForEmptyWords(current, "<s>")
            self.addAttributesForWords(current, sentence, wordIndex)
        for i in range(self.windowSize):
            if wordIndex + i + 1 < sentence.wordCount():
                self.addAttributesForWords(current, sentence, wordIndex + i + 1)
            else:
                self.addAttributesForEmptyWords(current, "</s>")



from MorphologicalAnalysis.MorphologicalParse import MorphologicalParse
from MorphologicalAnalysis.MorphologicalTag import MorphologicalTag
from DataGenerator.Attribute.LastIGContainsTagAttribute import LastIGContainsTagAttribute


class LastIGContainsTagAblativeAttribute(LastIGContainsTagAttribute):

    def __init__(self, parse: MorphologicalParse):
        """
        Binary attribute for a given word. If the last inflectional group of the word contains ABLATIVE tag,
        the attribute will be "true", otherwise "false".

        PARAMETERS
        ----------
        parse : MorphologicalParse
            Morphological parse of the word.
        """
        super().__init__(parse, MorphologicalTag.ABLATIVE)



from Classification.Attribute.BinaryAttribute import BinaryAttribute
from Dictionary.Word import Word


class IsOrganizationAttribute(BinaryAttribute):

    def __init__(self, surfaceForm: str):
        """
        Binary attribute for a given word. If the word is "corp.", "inc." or "co.", the attribute will have the
        value "true", otherwise "false".

        PARAMETERS
        ----------
        surfaceForm : str
            Surface form of the word.
        """
        super().__init__(Word.isOrganization(surfaceForm))



from Classification.Attribute.DiscreteAttribute import DiscreteAttribute
from MorphologicalAnalysis.MorphologicalParse import MorphologicalParse


class MainPosAttribute(DiscreteAttribute):

    def __init__(self, parse: MorphologicalParse):
        """
        Discrete attribute for a given word. Returns the last part of speech (main part of speech) of the word

        PARAMETERS
        ----------
        parse : MorphologicalParse
            Morphological parse of the word.
        """
        super().__init__(parse.getPos())



from DataGenerator.DatasetGenerator.DataSetGenerator import DataSetGenerator
from DataGenerator.InstanceGenerator.DisambiguationInstanceGenerator import DisambiguationInstanceGenerator


class DisambiguationDataSetGenerator(DataSetGenerator):

    def __init__(self, folder: str, pattern: str, disambiguationInstanceGenerator: DisambiguationInstanceGenerator):
        """
        Constructor for the DisambiguationDataSetGenerator which takes input the data directory, the pattern for the
        training files included, and an instanceGenerator. The constructor calls its super class with these inputs.

        PARAMETERS
        ----------
        folder : str
            Directory where the treebank files reside.
        pattern : str
            Pattern of the tree files to be included in the treebank. Use "." for all files.
        disambiguationInstanceGenerator : DisambiguationInstanceGenerator
            The instance generator used to generate the dataset.
        """
        super().__init__(folder, pattern, disambiguationInstanceGenerator)



from AnnotatedSentence.AnnotatedSentence import AnnotatedSentence
from AnnotatedSentence.AnnotatedWord import AnnotatedWord
from Classification.Instance.CompositeInstance import CompositeInstance
from Classification.Instance.Instance import Instance
from Corpus.Sentence import Sentence
from MorphologicalAnalysis.FsmMorphologicalAnalyzer import FsmMorphologicalAnalyzer
from WordNet.WordNet import WordNet

from DataGenerator.InstanceGenerator.SimpleWindowInstanceGenerator import SimpleWindowInstanceGenerator


class SemanticInstanceGenerator(SimpleWindowInstanceGenerator):

    __fsm: FsmMorphologicalAnalyzer
    __wordNet: WordNet

    def __init__(self, fsm: FsmMorphologicalAnalyzer, wordNet: WordNet):
        """
        Constructor for the semantic instance generator. Takes morphological analyzer and wordnet as input to set the
        corresponding variables.

        PARAMETERS
        ----------
        fsm : FsmMorphologicalAnalyzer
            Morphological analyzer to be used.
        wordNet : WordNet
            Wordnet to be used.
        """
        self.__fsm = fsm
        self.__wordNet = wordNet

    def addAttributesForWords(self, current: Instance, sentence: Sentence, wordIndex: int):
        """
        Generates a single classification instance of the WSD problem for the given word of the given sentence. If the
        word has not been labeled with sense tag yet, the method returns null. In the WSD problem, the system also
        generates and stores all possible sense labels for the current instance. In this case, a classification
        instance will not have all labels in the dataset, but some subset of it.

        PARAMETERS
        ----------
        sentence : Sentence
            Input sentence.
        wordIndex : int
            The index of the word in the sentence.

        RETURNS
        -------
        Instance
            Classification instance.
        """
        pass

    def addAttributesForEmptyWords(self, current: Instance, emptyWord: str):
        pass

    def generateInstanceFromSentence(self, sentence: Sentence, wordIndex: int) -> Instance:
        if isinstance(sentence, AnnotatedSentence):
            possibleSynSets = sentence.constructSynSets(self.__wordNet, self.__fsm, wordIndex)
            word = sentence.getWord(wordIndex)
            if isinstance(word, AnnotatedWord):
                classLabel = word.getSemantic()
                current = CompositeInstance(classLabel)
                possibleClassLabels = []
                for synSet in possibleSynSets:
                    possibleClassLabels.append(synSet.getId())
                current.setPossibleClassLabels(possibleClassLabels)
                self.addAttributes(current, sentence, wordIndex)
                return current



from Classification.Attribute.BinaryAttribute import BinaryAttribute
from NamedEntityRecognition.Gazetteer import Gazetteer


class IsLocationGazetteer(BinaryAttribute):

    gazetteer = Gazetteer("LOCATION", "gazetteer-location.txt")

    def __init__(self, surfaceForm: str):
        """
        Binary attribute for a given word. If the word is listed in the Location Gazetteer file, the attribute will
        have the value "true", otherwise "false".

        PARAMETERS
        ----------
        surfaceForm : str
            Surface form of the word.
        """
        super().__init__(self.gazetteer.contains(surfaceForm))



from Classification.Attribute.DiscreteAttribute import DiscreteAttribute


class SurfaceFormAttribute(DiscreteAttribute):

    def __init__(self, surfaceForm: str):
        """
        Discrete attribute for a given word. Returns the surface form.

        PARAMETERS
        ----------
        surfaceForm : str
            Surface form of the word.
        """
        super().__init__(surfaceForm)



from AnnotatedSentence.AnnotatedCorpus import AnnotatedCorpus
from Classification.DataSet.DataSet import DataSet

from DataGenerator.InstanceGenerator.InstanceGenerator import InstanceGenerator


class AnnotatedDatasetGenerator:

    __corpus: AnnotatedCorpus
    instanceGenerator: InstanceGenerator

    def __init__(self, folder: str, pattern: str, instanceGenerator: InstanceGenerator):
        """
        Constructor for the AnnotatedDataSetGenerator which takes input the data directory, the pattern for the
        training files included, and an instanceGenerator. The constructor loads the sentence corpus from the given
        directory including the given files having the given pattern.

        PARAMETERS
        ----------
        folder : str
            Directory where the corpus files reside.
        pattern : str
            Pattern of the tree files to be included in the treebank. Use "." for all files.
        instanceGenerator : InstanceGenerator
            The instance generator used to generate the dataset.
        """
        self.__corpus = AnnotatedCorpus(folder, pattern)
        self.instanceGenerator = instanceGenerator

    def setInstanceGenerator(self, instanceGenerator: InstanceGenerator):
        """
        Mutator for the instanceGenerator attribute.

        PARAMETERS
        ----------
        instanceGenerator : InstanceGenerator
            Input instanceGenerator
        """
        self.instanceGenerator = instanceGenerator

    def generate(self) -> DataSet:
        """
        Creates a dataset from the corpus. Calls generateInstanceFromSentence for each parse sentence in the corpus.

        RETURNS
        -------
        DataSet
            Created dataset.
        """
        dataSet = DataSet()
        for sentence in self.__corpus.sentences:
            for j in range(sentence.wordCount()):
                generatedInstance = self.instanceGenerator.generateInstanceFromSentence(sentence, j)
                if generatedInstance is not None:
                    dataSet.addInstance(generatedInstance)
        return dataSet



from setuptools import setup

from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text(encoding="utf-8")

setup(
    name='NlpToolkit-DataGenerator',
    version='1.0.6',
    packages=['DataGenerator', 'DataGenerator.Attribute', 'DataGenerator.CorpusGenerator',
              'DataGenerator.DatasetGenerator', 'DataGenerator.InstanceGenerator'],
    url='https://github.com/StarlangSoftware/DataGenerator-Py',
    license='',
    author='olcaytaner',
    author_email='olcay.yildiz@ozyegin.edu.tr',
    description='Classification dataset generator library for high level Nlp tasks',
    install_requires=['NlpToolkit-AnnotatedTree', 'NlpToolkit-Classification',
                      'NlpToolkit-MorphologicalDisambiguation'],
    long_description=long_description,
    long_description_content_type='text/markdown'
)



from AnnotatedSentence.AnnotatedSentence import AnnotatedSentence
from AnnotatedSentence.AnnotatedWord import AnnotatedWord
from AnnotatedSentence.AnnotatedCorpus import AnnotatedCorpus
from DisambiguationCorpus.DisambiguatedWord import DisambiguatedWord
from DisambiguationCorpus.DisambiguationCorpus import DisambiguationCorpus


class SentenceDisambiguationCorpusGenerator:

    __annotatedCorpus: AnnotatedCorpus

    def __init__(self, folder: str, pattern: str):
        """
        Constructor for the DisambiguationCorpusGenerator which takes input the data directory and the pattern for the
        training files included. The constructor loads the corpus from the given directory including the given files
        the given pattern.

        PARAMETERS
        ----------
        folder : str
            Directory where the sentence files reside.
        pattern : str
            Pattern of the tree files to be included in the corpus. Use "." for all files.
        """
        self.__annotatedCorpus = AnnotatedCorpus(folder, pattern)

    def generate(self) -> DisambiguationCorpus:
        """
        Creates a morphological disambiguation corpus from the corpus.

        RETURNS
        -------
        DisambiguationCorpus
            Created disambiguation corpus.
        """
        corpus = DisambiguationCorpus()
        for i in range(self.__annotatedCorpus.sentenceCount()):
            sentence = self.__annotatedCorpus.getSentence(i)
            disambiguationSentence = AnnotatedSentence()
            for j in range(sentence.wordCount()):
                annotatedWord = sentence.getWord(j)
                if isinstance(annotatedWord, AnnotatedWord):
                    disambiguationSentence.addWord(DisambiguatedWord(annotatedWord.getName(),
                                                                     annotatedWord.getParse()))
            corpus.addSentence(disambiguationSentence)
        return corpus



from AnnotatedTree.ParseTreeDrawable import ParseTreeDrawable
from AnnotatedTree.TreeBankDrawable import TreeBankDrawable
from Classification.DataSet.DataSet import DataSet

from DataGenerator.InstanceGenerator.InstanceGenerator import InstanceGenerator


class DataSetGenerator:

    __treeBank: TreeBankDrawable
    instanceGenerator: InstanceGenerator

    def __init__(self, folder: str, pattern: str, instanceGenerator: InstanceGenerator):
        """
        Constructor for the DataSetGenerator which takes input the data directory, the pattern for the training files
        included, and an instanceGenerator. The constructor loads the treebank from the given directory
        including the given files having the given pattern. If punctuations are not included, they are removed from
        the data.

        PARAMETERS
        ----------
        folder : str
            Directory where the treebank files reside.
        pattern : str
            Pattern of the tree files to be included in the treebank. Use "." for all files.
        instanceGenerator : InstanceGenerator
            The instance generator used to generate the dataset.
        """
        self.__treeBank = TreeBankDrawable(folder, pattern)
        self.instanceGenerator = instanceGenerator

    def setInstanceGenerator(self, instanceGenerator: InstanceGenerator):
        """
        Mutator for the instanceGenerator attribute.

        PARAMETERS
        ----------
        instanceGenerator : InstanceGenerator
            Input instanceGenerator
        """
        self.instanceGenerator = instanceGenerator

    def generateInstanceListFromTree(self, parseTree: ParseTreeDrawable) -> list:
        """
        The method generates a set of instances (an instance from each word in the tree) from a single tree. The method
        calls the instanceGenerator for each word in the sentence.

        PARAMETERS
        ----------
        parseTree : ParseTreeDrawable
            Parsetree for which a set of instances will be created

        RETURNS
        -------
        list
            A list of instances.
        """
        instanceList = []
        annotatedSentence = parseTree.generateAnnotatedSentence()
        for i in range(annotatedSentence.wordCount()):
            generatedSentence = self.instanceGenerator.generateInstanceFromSentence(annotatedSentence, i)
            if generatedSentence is not None:
                instanceList.append(generatedSentence)
        return instanceList

    def generate(self) -> DataSet:
        """
        Creates a dataset from the treeBank. Calls generateInstanceListFromTree for each parse tree in the treebank.

        RETURNS
        -------
        DataSet
            Created dataset.
        """
        dataSet = DataSet()
        for i in range(self.__treeBank.size()):
            parseTree = self.__treeBank.get(i)
            dataSet.addInstanceList(self.generateInstanceListFromTree(parseTree))
        return dataSet
